module Task1 where

import Data.Char (toLower)
import Data.List (sort, sortOn, group)

-- Task 1:
-- Given a number n and a string s containing English words
-- (and nothing else), generate a report that lists the n
-- most common words in the given string s.

-- input string s: "see Spot run  run spot run"
-- input number n: 2
-- 

-- 1. break the input string into words
breakIntoWords :: String -> [String]
breakIntoWords = words
-- ["see","Spot","run","run","spot","run"]

-- 2. convert the words to lowercase
convertToLowercase :: [String] -> [String]
convertToLowercase = map (map toLower)
-- ["see","spot","run","run","spot","run"]

-- 3. sort the words
sortWords :: [String] -> [String]
sortWords = sort
-- ["run","run","run","see","spot","spot"]

-- 4. group adjacent occurrences (runs)
type Run = [String]
groupAdjacentRuns :: [String] -> [Run]
groupAdjacentRuns = group
-- [["run","run","run"],["see"],["spot","spot"]]
    --3                 --1     --2
-- 5. sort by length of the run
sortByLength :: [Run] -> [Run]
sortByLength = sortOn length
-- [["see"], ["spot","spot"], ["run","run","run"]]

-- 6. take the n longest runs
takeLongestRuns :: Int -> [Run] -> [Run]
takeLongestRuns n runs = take n (reverse runs)

-- 7. generate report
generateReport :: [Run] -> String
generateReport runs = unwords (map displayRun runs) where
  displayRun :: Run -> String
  displayRun run = theWord ++ " " ++ show theTimes where
    theWord :: String
    theWord = head run
    theTimes :: Int
    theTimes = length run

-- then put it all together
program :: Int -> String -> String
program n =
  generateReport     .
  takeLongestRuns n  .
  sortByLength       .
  groupAdjacentRuns  .
  sortWords          .
  convertToLowercase .
  breakIntoWords

-- Conditionals:

polarity :: Int -> String
polarity x = if (x > 0) then "positive"
                            else if (x < 0) then "negative"
                                            else "zero"


polarity2 :: Int -> String
polarity2 x
  | x > 0   = "positive"
  | x < 0   = "negative"
  | otherwise = "zero"

polarity3 :: Int -> String
polarity3 x =
  case x > 0 of
    False -> case x < 0 of
      False -> "zero"
      True -> "negative"
    True  -> "positive"

-- We can rewrite simple for loops
-- into recursive functions.

{-
  int fact(int n) {
    int result = 1;
    for (int i = 1; i <= n; i++) {
      result = result * i;
    }
    return result;
  }
-}

-- First enumerate the variables that occur:
-- int n
-- int result
-- int i

-- one argument for each variable
fact' :: Int -> Int -> Int -> Int
fact' n result i
  | i <= n = fact' n (result * i) (i + 1)
  | otherwise = result

fact :: Int -> Int
fact n = fact' n 1 1

-- the Haskell way:

factorial :: Int -> Int
factorial n = product [1..n]

-- pattern matching, recursion

isEmpty :: [a] -> Bool
isEmpty xs = case xs of
  []     -> True
  (y:ys) -> False

isEmpty' :: [a] -> Bool
isEmpty' [] = True
isEmpty' (y:ys) = False


product' :: [Int] -> Int
product' [] = 1
product' (y:ys) = y * product' ys
    -- y is an Int
    -- ys is a (potentially an empty) list of Ints
